<?php

namespace App\Controller\OgRepository\Webhook;

use App\Entity\Image;
use App\Entity\Trace;
use App\Model\ImageStatus;
use App\Model\TraceStatus;
use Doctrine\ORM\EntityManagerInterface;
use Psr\Log\LoggerInterface;
use Symfony\Bundle\FrameworkBundle\Controller\AbstractController;
use Symfony\Component\HttpFoundation\JsonResponse;
use Symfony\Component\HttpFoundation\Request;
use Symfony\Component\HttpFoundation\Response;
use Symfony\Component\HttpKernel\Attribute\AsController;
use Symfony\Component\Routing\Annotation\Route;

#[AsController]
class ResponseController extends AbstractController
{
    public function __construct(
        protected readonly EntityManagerInterface   $entityManager,
        protected readonly LoggerInterface          $logger,
    )
    {
    }

    #[Route('/og-repository/webhook', name: 'og_repository_webhook', methods: ['POST'])]
    public function repositoryWebhook(Request $request): JsonResponse
    {
        $data = json_decode($request->getContent(), true);

        if (!isset($data['job_id'])) {
            return new JsonResponse(['message' => 'Invalid request'], Response::HTTP_BAD_REQUEST);
        }

        $action = $data['job_id'];

        if (str_starts_with($action, "CreateAuxiliarFiles_")) {
            $this->handleCreateAuxFiles($action, $data);
        } elseif (str_starts_with($action, "ExportImage_")) {
            $this->handleExportImage($action, $data);
        } elseif (str_starts_with($action, "ImportImage_")) {
            $this->handleImportImage($action, $data);
        } else {
            return new JsonResponse(['message' => 'Invalid action'], Response::HTTP_BAD_REQUEST);
        }

        return new JsonResponse($data, Response::HTTP_OK);
    }

    private function handleCreateAuxFiles(string $action, array $data): void
    {
        $trace = $this->entityManager->getRepository(Trace::class)->findOneBy(['jobId' => $data['job_id']]);
        $imageUuid = $trace->getInput()['imageUuid'];

        $image = $this->entityManager->getRepository(Image::class)->findOneBy(['uuid' => $imageUuid]);

        if ($image === null) {
            $trace->setStatus(TraceStatus::FAILED);
            $trace->setFinishedAt(new \DateTime());
            $trace->setOutput('Image not found');
            $this->entityManager->persist($trace);
            $this->entityManager->flush();

            new JsonResponse(['message' => 'Image not found'], Response::HTTP_NOT_FOUND);
            return;
        }

        $image->setImageFullsum($data['image_id']);
        $image->setStatus(ImageStatus::SUCCESS);
        $this->entityManager->persist($image);

        $trace->setStatus(TraceStatus::SUCCESS);
        $trace->setFinishedAt(new \DateTime());

        $this->entityManager->persist($trace);
        $this->entityManager->flush();
    }

    private function handleExportImage(string $action, array $data): void
    {
        $trace = $this->entityManager->getRepository(Trace::class)->findOneBy(['jobId' => $data['job_id']]);

        $imageUuid = $trace->getInput()['imageUuid'];

        $image = $this->entityManager->getRepository(Image::class)->findOneBy(['uuid' => $imageUuid]);

        if ($image === null) {
            $trace->setStatus(TraceStatus::FAILED);
            $trace->setFinishedAt(new \DateTime());
            $trace->setOutput('Image not found');
            $this->entityManager->persist($trace);
            $this->entityManager->flush();

            new JsonResponse(['message' => 'Image not found'], Response::HTTP_NOT_FOUND);
            return;
        }

        $this->logger->info('Image exported', ['image' => $image->getName()]);

        $image->setStatus(ImageStatus::SUCCESS);
        $this->entityManager->persist($image);


        $trace->setStatus(TraceStatus::SUCCESS);
        $trace->setFinishedAt(new \DateTime());

        $this->entityManager->persist($trace);
        $this->entityManager->flush();
    }

    private function handleImportImage(string $action, array $data): void
    {
        $trace = $this->entityManager->getRepository(Trace::class)->findOneBy(['jobId' => $data['job_id']]);

        $imageUuid = $trace->getInput()['imageUuid'];

        $image = $this->entityManager->getRepository(Image::class)->findOneBy(['uuid' => $imageUuid]);

        if ($image === null) {
            $trace->setStatus(TraceStatus::FAILED);
            $trace->setFinishedAt(new \DateTime());
            $trace->setOutput('Image not found');
            $this->entityManager->persist($trace);
            $this->entityManager->flush();

            new JsonResponse(['message' => 'Image not found'], Response::HTTP_NOT_FOUND);
            return;
        }

        $this->logger->info('Image imported', ['image' => $image->getName()]);

        $image->setStatus(ImageStatus::SUCCESS);
        $this->entityManager->persist($image);


        $trace->setStatus(TraceStatus::SUCCESS);
        $trace->setFinishedAt(new \DateTime());

        $this->entityManager->persist($trace);
        $this->entityManager->flush();
    }
}